﻿;===Description=========================================================================
/*



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MODIFIED serously
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



[function] BarChart
Author:		Learning one (Boris Mudrinić)
Contact:	boris.mudrinic@gmail.com
Version:	1.01
Link:		www.autohotkey.com/community/viewtopic.php?f=2&t=89246

Requires Gdip.ahk by Tic; www.autohotkey.com/community/viewtopic.php?f=2&t=32238
Compatible with AHK_L and AHK Basic

====== License ======
You can non-commercialy use and redistribute BarChart if put 2 things in your product's documentation or "About MsgBox"; 1) credit to the author, 2) link to BarChart's AutoHotkey forum post. Author is not responsible for any damages arising from use or redistribution of his work. If redistributed in source form, you are not allowed to remove comments from this file. If you are interested in commercial usage, contact the author to get permission for it.
*/



;===Function============================================================================
BarChart(inputData, Width, Height="", TitleText="", Skin="LightOrange", Options="") {		

BarColorDirection := 0, BarColorWidthDiv := 1, BarColorHeightDiv := 1
TitleBackColorA := 0, TitleBackColorB := 0, TitleBackColorDirection := 1, TitleBorderColor := 0, TitleTextColor := "ff6B624C", TitleHeightFactor := 0
PlotBackColorA := 0, PlotBackColorB := 0, PlotBackColorDirection := 1, PlotRangeBorderColor := 0

ChartBackColorA := "ffFFFFFF", ChartBackColorB := "ffFFFFFF", ChartBackColorDirection := 1
ChartBackHatchColorA := 0, ChartBackHatchColorB := 0, ChartBackHatchStyle := 18, ChartBackRoundness := 0
	
	;=== if user wants to automaticaly set bitmap to picture control and than dispose of it ===
	if Width is not number
	{
		; In this case width can be: 1) name of the control's associated variable or 2) a string "hwnd" followed by the handle of the control
		SetBitmap2Control := 1
		ControlID := Width
		GuiNum := (Height = "") ? 1 : Height	; in this case, Height parameter represents Gui number
		if (SubStr(Width,1,4) = "hwnd") {	; extra feature for AHK_L users - use  handle of the control
			PotentialHwnd := SubStr(Width,5)
			if PotentialHwnd is number
				hControl := PotentialHwnd, ControlID := PotentialHwnd	; [AHK_L v1.1.04+]: ControlID can be the HWND of the control
			else
				GuiControlGet, hControl, %GuiNum%:hwnd, %ControlID%
		}
		else		
			GuiControlGet, hControl, %GuiNum%:hwnd, %ControlID%
			
		GuiControlGet, Control, %GuiNum%:Pos, %ControlID%	; get control's Width & Height
		Width := ControlW, Height := ControlH				; store control's Width & Height
	}
	
	;=== User's options & overrides ===	
	Loop, parse, Options, `,, %A_Space%
		colonpos := InStr(A_LoopField, ":"), var := SubStr(A_LoopField, 1, colonpos-1), val := SubStr(A_LoopField, colonpos+1), %var% := val

	;=== Other options, defaults ===	
	
	BarSpacing := 1
	BarHeightFactor := 1
	
	TextSize := (TextSize = "") ? 12 : TextSize
	TextFont := (TextFont = "") ? "Arial" : TextFont
		
	BarRoundness := 0 
	TextIndentation := (TextIndentation = "") ? 4 : TextIndentation
	RowsDelimiter := (RowsDelimiter = "") ? "`n" : RowsDelimiter
	ColumnsDelimiter := (ColumnsDelimiter = "") ? "`t" : ColumnsDelimiter
	DisplayValues := (DisplayValues = "") ? 1 : DisplayValues
	DataValueSeparator := (DataValueSeparator = "") ? ": " : DataValueSeparator
	DataAlignment := (DataAlignment = "") ? "Left" : DataAlignment						; Left,Right,Center. Note: case sensitive!
	MaxValue := (MaxValue = "") ? 0 : MaxValue
	TextRendering := 0
	TitleTextSize := 0
	TitleTextFormat := (TitleTextFormat = "") ? "Bold" : TitleTextFormat				; Available formats: "Regular|Bold|Italic|BoldItalic|Underline|Strikeout". Note: case sensitive!
	TitleAlignment := "Left"		; Left, Right, Center. Note: case sensitive!
	TitleIndentation := (TitleIndentation = "") ? 0 : TitleIndentation
	FrameWidth := 0
	SmoothingMode := 0 

	;=== Bitmap, Graphics, SmoothingMode ===
	pBitmap := Gdip_CreateBitmap(Width, Height)
	if !pBitmap																											; than user probably didn't start up GDI+
	{
		if SetBitmap2Control																							; do the job for the user
			pToken := Gdip_Startup(), pBitmap := Gdip_CreateBitmap(Width, Height)
		else																													; don't allow this case
			return
	}
	G := Gdip_GraphicsFromImage(pBitmap), Gdip_SetSmoothingMode(G, SmoothingMode)
	
	;=== Get Bar height ===
	TextMeasure := Gdip_TextToGraphics(G, "T_", "x0 y0 Vcenter " DataAlignment " c" BarTextColor " r" TextRendering " s" TextSize, TextFont, Width, Height, 1)
	StringSplit, m, TextMeasure, |
	TextHeight := m4
	BarHeight := TextHeight * BarHeightFactor
	
	TitleHeight := 0
	
	;=== If only one column in inputData ===
	if (InStr(inputData, ColumnsDelimiter) = 0) {
		Loop, parse, inputData, %RowsDelimiter%
			NewinputData .= ColumnsDelimiter A_LoopField RowsDelimiter
		inputData := SubStr(NewinputData, 1, StrLen(NewinputData) - StrLen(RowsDelimiter))
	}
	
	;=== Get dimensions, layout... ===
	TotalBars := 0, MaxPositiveValues := 0, MaxNegativeValues := 0
	Loop, parse, inputData, %RowsDelimiter%
	{
		TotalBars++
		if A_LoopField is Space
			continue
		StringSplit, Field, A_LoopField, %ColumnsDelimiter%
		if Field2 is not number
			continue
		if (Field2 > MaxPositiveValues)
			MaxPositiveValues := Field2
		if (Field2 < MaxNegativeValues)
			MaxNegativeValues := Field2
		if Field2 is not number
			field1 := "", field2 := ""
	}
	MaxValue := abs(MaxValue)
	if (MaxPositiveValues > 0 and MaxNegativeValues = 0) { ; only positive
		inputDataMaxRange := (MaxValue > MaxPositiveValues) ? MaxValue : MaxPositiveValues
	}
	else if (MaxPositiveValues = 0 and MaxNegativeValues < 0) { ; only negative
		inputDataMaxRange := (MaxValue > abs(MaxNegativeValues)) ? - MaxValue : MaxNegativeValues
	}
	else if (MaxPositiveValues > 0 and MaxNegativeValues < 0) { ; positive & negative
		Candidates := MaxValue "`n" MaxPositiveValues "`n" Abs(MaxNegativeValues)
		inputDataMaxRange := 0
		Loop, parse, Candidates, `n
			if (A_LoopField > inputDataMaxRange)
				inputDataMaxRange := A_LoopField
		inputDataMaxRange *= 2
		DataAlignment := "Center", TitleAlignment := "Center"						; force to Center data and title alignment
	}
	else	; zeros
		inputDataMaxRange := 1
	TotalBarSpacing := (TotalBars-1)*BarSpacing
	MaxBarWidth := Width - FrameWidth*2
	BarUnitWidth := abs(MaxBarWidth/inputDataMaxRange)
	if (TitleText != "")
		y := (Height-(BarHeight*TotalBars+TotalBarSpacing+TitleHeight+BarSpacing))/2
	else
		y := (Height-(BarHeight*TotalBars+TotalBarSpacing))/2
	
	;=== Draw Chart back ===
	pBrush := Gdip_CreateLineBrushFromRect(0, 0, Width, Height, "0x" ChartBackColorA, "0x" ChartBackColorB, ChartBackColorDirection, 1)
	Gdip_FillRoundedRectangle(G, pBrush, 0, 0, Width, Height, ChartBackRoundness)
	Gdip_DeleteBrush(pBrush)
	pBrush := Gdip_BrushCreateHatch("0x" ChartBackHatchColorA, "0x" ChartBackHatchColorB, ChartBackHatchStyle)
	Gdip_FillRoundedRectangle(G, pBrush, 0, 0, Width, Height, ChartBackRoundness)
	Gdip_DeleteBrush(pBrush)
	
	;=== Draw Title area ===
	if (TitleText != "")
	{
		if (TitleBackColorA != 0 or TitleBackColorB != 0) {
			pBrush := Gdip_CreateLineBrushFromRect(FrameWidth, y, MaxBarWidth, TitleHeight, "0x" TitleBackColorA, "0x" TitleBackColorB, TitleBackColorDirection, 1)
			Gdip_FillRoundedRectangle(G, pBrush, FrameWidth, y, MaxBarWidth, TitleHeight, BarRoundness)
			Gdip_DeleteBrush(pBrush)
		}
		if  (TitleBorderColor != 0) {
			pPen := Gdip_CreatePen("0x" TitleBorderColor, 1)
			Gdip_DrawRoundedRectangle(G, pPen, FrameWidth, y, MaxBarWidth, TitleHeight, BarRoundness)
			Gdip_DeletePen(pPen)
		}
		Gdip_TextToGraphics(G, TitleText, "x" FrameWidth + TitleIndentation " y" y+1 " Vcenter " TitleAlignment " "  TitleTextFormat " c" TitleTextColor " r" TextRendering " s" TitleTextSize
			, TextFont, MaxBarWidth - TitleIndentation*2, TitleHeight)
		y := y+TitleHeight+BarSpacing
	}
	
	;=== Draw Plot area ===
	if (PlotBackColorA != 0 or PlotBackColorB != 0) {
			pBrush := Gdip_CreateLineBrushFromRect(FrameWidth, y, MaxBarWidth, BarHeight*TotalBars+TotalBarSpacing, "0x" PlotBackColorA, "0x" PlotBackColorB, PlotBackColorDirection, 1)
			Gdip_FillRoundedRectangle(G, pBrush, FrameWidth, y, MaxBarWidth, BarHeight*TotalBars+TotalBarSpacing, BarRoundness)
			Gdip_DeleteBrush(pBrush)
		}
	if (PlotRangeBorderColor != 0)	{
		pPen := Gdip_CreatePen("0x" PlotRangeBorderColor, 1)
		Gdip_DrawRoundedRectangle(G, pPen, FrameWidth, y, MaxBarWidth, BarHeight*TotalBars+TotalBarSpacing, BarRoundness)		
		Gdip_DeletePen(pPen)
	}
	
	;=== Draw Bars & Text ===
	
	y := y - BarHeight - BarSpacing	
	y := 0
	
	Loop, parse, inputData, %RowsDelimiter%
	{
		y := y + BarHeight + BarSpacing
		if A_LoopField is Space		; bar separator
			continue
		StringSplit, Field, A_LoopField, %ColumnsDelimiter%
		
		if Field1 is Space
			Field1 := ""
		if Field2 is not number
			Field2 := ""
			
		If(Field2 = 0)								
			Field1 := "", Field2 := ""

		if (MaxPositiveValues > 0 and MaxNegativeValues < 0) { 
			if (Field2 >= 0)
				x := Width/2
			else
				x := Width/2 - BarUnitWidth * abs(Field2)
			DivW := 2
		}
		else {
			x := FrameWidth
			DivW := 1
		}
		
		if (Field2 < 30)
			Color := "ffAA853F", BarTextColor := "ff000000"
		else if (Field2 < 70)
			Color := "ffFFA500", BarTextColor := "ff000000"
		else if (Field2 < 100)
			Color := "ff3CB371", BarTextColor := "ffFFFFFF"
		else
			Color := "ff1E60FF", BarTextColor := "ffFFFFFF"
			
		BarColorA := "33" Color, BarColorB := "ff" Color, BarBorderColor := "ee" Color
		pBrush := Gdip_CreateLineBrushFromRect(x, y, MaxBarWidth/BarColorWidthDiv/DivW, BarHeight/BarColorHeightDiv, "0x" BarColorA, "0x" BarColorB, BarColorDirection, 1)
		Gdip_FillRoundedRectangle(G, pBrush, x, y, BarUnitWidth*abs(Field2), BarHeight, BarRoundness)
		Gdip_DeleteBrush(pBrush)
		if (DisplayValues = 1) 
		{
			if (Field1 = "" and Field2 = "")
				BarText := ""
			else if (Field1 != "" and Field2 = "")
				BarText := Field1
			else if (Field1 = "" and Field2 != "")
				BarText := Field2 "%"
			else if (Field1 != "" and Field2 != "")
				BarText := Field1 DataValueSeparator Field2 "%"
		}
		else
			BarText := Field1
		;																			     글자정렬좌우
		Gdip_TextToGraphics(G, BarText, "x" FrameWidth " y" y+1 " Center c" BarTextColor " r" TextRendering " s" TextSize, TextFont, MaxBarWidth, BarHeight)
		field1 := "", field2 := ""
	}
	
	Gdip_DeleteGraphics(G)
	if SetBitmap2Control
	{
		hBitmap := Gdip_CreateHBITMAPFromBitmap(pBitmap), SetImage(hControl, hBitmap), DeleteObject(hBitmap)	
		GuiControl, %GuiNum%:MoveDraw, %ControlID%								; repaints the region of the GUI window occupied by the control
		Gdip_DisposeImage(pBitmap)
		if pToken																								; if this function started up GDI+, than shut it down
			Gdip_Shutdown(pToken)
	}
	else	
		return pBitmap
}
